'' =================================================================================================
''
''   File....... jm_sbus_rx_test.spin
''   Purpose.... Receives and displays data from Futaba S.BUS stream
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2021 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon.mcphalen@gmail.com
''   Started....
''   Updated.... 25 MAR 2021
''
''   {$P1}
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { terminal }

  BR_TERM  = 115_200                                            ' terminal baud rate

  #0, T_PST, T_ANSI                                             ' terminal types

  T_TYPE = T_PST


con { fixed io pins }

  PGM_RX  = 31  { I }                                           ' serial / programming
  PGM_TX  = 30  { O }

  EE_SDA  = 29  { I/O }                                         ' i2c / eeprom
  EE_SCL  = 28  { I/O }


con { app io pins }

  SBUS_RX = 27  { I }                                           ' S.BUS serial input


con

  #true,  ON, OFF
  #false, NO, YES


obj

' main                                                          ' * master Spin cog
  time : "jm_time_80"                                           '   timing and delays (80MHz)
  sbus : "jm_sbus_rx"                                           ' * s.bus serial input/decoding
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
  ansi : "jm_ansi"                                              '   ANSI terminal control sequences

' * uses cog when loaded


dat

' Default values from S.BUS object until receiver provides data

  Defaults      word    1023, 1023,    0, 1023, 1023, 1023, 1023, 1023, 1023
                word    1023, 1023, 1023, 1023, 1023, 1023, 1023,    0,    0


var { globals }


pub main | t, ch

  setup

  wait_for_terminal(true)
  time.pause(100)

  term.str(string("P1 S.BUS Input Test"))

  ' This debug loop is slow because of the fancy formatting

  t := cnt
  repeat
    show_status(0, 2)
    show_channels(0, 4)
    show_packets(0, 24)
    waitcnt(t += (100 * MS_001))


dat

  s_Live        byte    "-- Signal: Live  ", 0
  s_Lost        byte    "-- Signal: Lost  ", 0
  s_Failsafe    byte    "-- Failsafe mode ", 0


pub show_status(x, y)

  move_cursor(x, y)

  if (sbus.has_signal)
    term.str(@s_Live)
  else
    if (sbus.in_failsafe)
      term.str(@s_Failsafe)
    else
      term.str(@s_Lost)


pub show_channels(x, y) | ch

  move_cursor(x, y)

  repeat ch from 1 to 18
    term.fstr2(string("%2d  %4d \r"), ch, sbus.read(ch))


pub show_packets(x, y) | ch

  move_cursor(x, y)

  term.fstr1(string("Packets received: %d  "), sbus.count)


pub setup

'' Setup IO and objects for application

  time.start                                                    ' setup timing & delays

  sbus.start(SBUS_RX, @Defaults)                                ' start s.bus rx *

  term.tstart(BR_TERM)                                          ' start serial for terminal *


pub wait_for_terminal(clear)

'' Wait for terminal to be open and key pressed

  term.rxflush
  term.rx
  if (clear)
    clear_screen


pub clear_screen

  if (T_TYPE == T_PST)
    term.tx(term#CLS)
  else
    term.str(ansi.hide_cursor)
    term.str(ansi.home)
    term.str(ansi.cls)


pub move_cursor(x, y)

  if (T_TYPE == T_PST)
    term.tx(term#CRSR_XY)
    term.tx(x)
    term.tx(y)
  else
    term.str(ansi.crsr_xy(x, y))


con { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}